
 void freeRequestData(RequestData * data){
 	memset(&data, 0, sizeof(struct RequestData));
	}
	
//This function should have a virgin copy of RequestData and DataBlockPointer struct, meaning:
//No space should be allocated!
//NOTE: this function will allocate at least 2 bytes a pointer, located in the structs.
//      this enables every memory address to be deallocated as an array (to make my life easier).
//What does it do:
    //It takes the input from buffer, and puts everything into the RequestData
    //structured form.  It takes care of any allocation.


//NOTE: This function will be kinda like reentrant strtok, but not quite (will support returning empty lines, hooray)!
//Usage: first call pass buffer, and a unused pointer, after that, just keep passing pointer, and set buffer to NULL!
///GOAL: to support as much tolerance as listed in appendix 19.3 of the HTTP/1.1 RFC

//Takes the query off the url
int extractquery(char * uri, char ** url, char ** query){
    char * queryToken;
    char * pchar;
    
    pchar = strtok_r(uri, "?#", &queryToken);
    if(pchar == NULL)
    	return ESYNTAX;
    *url = (char *)malloc((strlen(pchar) + 1) * sizeof(char));
    if(*url == NULL)
    	return EMALLOC;
    *url[0] = 0;
    strcpy(*url, pchar);
    
    
    //for "#anchor" style support, Almost forgot...
    pchar = strtok_r(NULL, "#", &queryToken);
    if(pchar == NULL){
        *query = NULL;
    	return 0;
	}
    *query = (char *)malloc((strlen(pchar) + 1) * sizeof(char));
    if(*query == NULL)
    	return EMALLOC;
    *query[0] = 0;
    strcpy(*query, pchar);
    return 0;
    }
    
    
//Pass in line like "GET / HTTP/1.1", note that allocation will be done for you!
int httplinerequest(char * line, char ** method, char ** url, char **query, char ** protocol){
     char * pToken;     
     char * tstrtok;
     
     //Get Method(GET, POST, etc)
     pToken = strtok_r(line, " \r\n", &tstrtok);
     if(pToken == NULL)
	return ESYNTAX;	
     *method = (char *)malloc((strlen(pToken) + 1) * sizeof(char));
     if(method == NULL)
     	return EMALLOC;
     *(*method + 0) = 0;	
     strcpy(*method, pToken);
     
     //Get requestURI
     pToken = strtok_r(NULL, " \r\n", &tstrtok);
     if(pToken == NULL)
     	return ESYNTAX;

     {
     int returnval = extractquery(pToken, &(*url), &(*query));
     if(returnval != 0)
     	return returnval;
        
        
     //TODO: Simplify URL (ie "/cgi-bin/../cgi-bin/index.cgi" is "/cgi-bin/index.cgi"
     returnval = testmalicious(*url);
     if(returnval != 0)
     	return HE_BAD_REQUEST;
     }
     
     //get HTTP protocol
     pToken = strtok_r(NULL, " \r\n", &tstrtok);
     
     //protocol, otherwise use HTTP/1.0
     if(pToken != NULL){
        *protocol = (char *)malloc((strlen(pToken) + 1) * sizeof(char));
	if(protocol == NULL)
     	    return EMALLOC;
        *(*protocol + 0) = 0;
        strcpy(*protocol, pToken);
	}
     else{//fill protocol with assumed HTTP/1.0, with NULL
        *protocol = NULL;
	}
	
	return 0;
	}
     
//simply looks for :, and seperates, returns ESYNTAX if there is a problem
int GeneralHeader(char *header, char ** name, char ** value){
	char *colonTok;
	char *tPointer;
	
	
	tPointer = strtok_r(header, ":", &colonTok);
	if(tPointer == NULL)
		return ESYNTAX;
	*name = (char *)malloc((strlen(tPointer) + 1) * sizeof(char));
	if(*name == NULL){
		*value = NULL;
		return EMALLOC; 
		}
	strcpy(*name, tPointer);	
	
	if(colonTok == NULL){
		*value = NULL;
		return ESYNTAX;
		}
	*value = (char *)malloc((strlen(colonTok) + 1) * sizeof(char));
	if(*value == NULL)
		return EMALLOC; 
	strcpy(*value, colonTok);	
	return 0;
	}
  

	


//Buffer will be allocated with malloc, remember to free!
//Will stop at first empty line
//Connection must already be open!
//Each of these called will take about 4KB space. . .
int getRequest(char ** buffer, int socketd, int mem = 1024){
        char *tempbuffer = (char *)malloc( mem * sizeof(char) );
	if(tempbuffer == NULL){
		*buffer = NULL;
		return EMALLOC;
		}
	tempbuffer[0] = 0;
		
	int datac = 0;	
	int treturn; 
	int dc = 0;
	int endlCount = 0;
	//int returnval;
	
	
	char *tRecv = (char *)malloc(2 * sizeof(char));
	if(tRecv == NULL){
		*buffer = NULL;
		return EMALLOC;
		}
	tRecv[1] = 0;
	
	
	for( ; dc < mem - 1 ; dc++){
		treturn = recv(socketd, tRecv, 1, 0);
		
		    
		if(treturn == 0){
		  free(tRecv);
		  free(tempbuffer);
		  return ECONNECTION;
		  }
		if(treturn == -1){
		        free(tempbuffer);
			return ECONNECTION;
			}
		
		if(tRecv[0] == '\x0A'){
		    if(endlCount == 1)
		       break;
		    endlCount++;
		    }
		else if(tRecv[0] == '\x0D'){}
		else
		    endlCount = 0;
		   
			
		strcat(tempbuffer, tRecv);
		datac++;
		}
	*buffer = (char *)malloc((strlen(tempbuffer) + 1) * sizeof(char));
	if(*buffer == NULL){
	    free(tRecv);
	    free(tempbuffer);
	    return EMALLOC;
	    }
		
	strcpy(*buffer, tempbuffer);
	free(tRecv);
	free(tempbuffer);
	return 0;
	}
	
//MEM = 0 means unlimited, not that great of an idea
//NOTE: normal POST non-chunked data will always be loaded into the memory, buffer
int getRequestData(char ** buffer, int socketd, size_t dataSize, size_t dataLimit = 0){
	size_t curcnt = 0;
	int returnval= 0;
	char *tRecv = (char *)malloc(2 * sizeof(char));
	if(tRecv == NULL){
		*buffer = NULL;
		return EMALLOC;
		}
	tRecv[1] = 0;
	
	while(curcnt < dataSize){
	      returnval = recv(socketd, tRecv, 1, 0);
	      if(returnval != 1){
	      	 free(tRecv);
		 return ECONNECTION;
		 }
	      returnval = racat(&(*buffer), tRecv);	
	      if(returnval)
	      	break;
	      curcnt++;
	      if(dataLimit != 0){
	      	if(dataLimit <= curcnt)
			break;
		}
	      }
	free(tRecv);
	return returnval;
	}	

	/*
//MEM = 0 means unlimited, not that great of an idea
//NOTE: chunked data will always be written to a temporary file, filename!
int getChunkedRequestData(char ** buffer, int socketd, size_t dataLimit = 0){
	size_t curcnt = 0;
	int returnval= 0;
	char *tRecv = (char *)malloc(2 * sizeof(char));
	char *HexChunk = NULL;
	if(tRecv == NULL){
		*buffer = NULL;
		return EMALLOC;
		}
	tRecv[1] = 0;
	
	while(curcnt < dataLimit){
	      returnval = recv(socketd, tRecv, 1, 0);
	      if(returnval != 1){
	      	 free(tRecv);
		 return ECONNECTION;
		 }
		
		 if(tRecv == " "){
		 	
			}
		racat(&HexChunk, tRecv);
		
		
		
		getRequestData(&(*buffer), socketd, sizehere, 0);
		
		curcnt++;
	      	if(dataLimit != 0){
	      	      if(dataLimit <= curcnt)
			break;
			}
		}
	
	
	free(tRecv);
	
	return 0;
	}*/	
	
	
int Request(int socketd, struct RequestData * pRequestStruct, struct serverConf *serverconf){
     //Remember! pReuestStruct should be virgin, or at least completly unallocated!
     //All functions used here will do all allocation, or will set value to NULL
  
  memset(pRequestStruct, 0, sizeof(RequestData));//NULL out struct, so it's safe to delete!   
     
  int returnval;
  char * request;
  returnval = getRequest(&request, socketd, serverconf->maxheaderdata);

  if(returnval){
  	cout << "getRequest error...\n";
  	return returnval;
	}
	
     char * tPointer;
     char * LineToken;
     

     
     tPointer = strtok_r(request, "\x0A\x0D", &LineToken);
     if(tPointer == NULL)
     	return ESYNTAX;
    returnval = httplinerequest(tPointer, &pRequestStruct->method, &pRequestStruct->url, &pRequestStruct->query, &pRequestStruct->protocol);
    if(returnval){
	cout << "httplinerequest error...\n";
	if(returnval == ESYNTAX)
    		return HE_BAD_REQUEST;
	else
    		return returnval;
	}  
      
     char *name;
     char *value;
      
     while(tPointer != NULL){
         tPointer = strtok_r(NULL, "\x0A\x0D", &LineToken);
	 if(tPointer == NULL)
	 	break;
		
	 returnval = GeneralHeader(tPointer, &name, &value);
	 
	 if(returnval){
	 	cout << "GeneralHeader error...\n";
	 	free(name);
	 	free(value);
		return returnval;
		}
	 
	 returnval = removefirst(&value, " ");
	 
	 if(returnval){
	 	cout << "removefirst error...\n";
	 	free(name);
	 	free(value);
		return returnval;
		}
	 
		
	//***********   STRING   ***********
	 if(strcasecmp(name, "host") == 0){
	       racpy(&pRequestStruct->host, value);
	       }
	 if(strcasecmp(name, "user-agent") == 0){
	       racpy(&pRequestStruct->useragent, value);
	       }
	 if(strcasecmp(name, "referer") == 0){
	       racpy(&pRequestStruct->referer, value);
	       cout << "REFERER:" << pRequestStruct->referer << "\n";
	       }
	 if(strcasecmp(name, "range") == 0){
	       racpy(&pRequestStruct->range, value);
	       }
	 if(strcasecmp(name, "content-type") == 0){
	       racpy(&pRequestStruct->range, value);
	       }
	 if(strcasecmp(name, "connection") == 0){
	       racpy(&pRequestStruct->connection, value);
	       }
	 if(strcasecmp(name, "accept") == 0){
	       racpy(&pRequestStruct->accept, value);
	       }
	 if(strcasecmp(name, "cookie") == 0){
	       racpy(&pRequestStruct->cookie, value);
	       }
	 if(strcasecmp(name, "from") == 0){
	       racpy(&pRequestStruct->from, value);
	       }
	 if(strcasecmp(name, "transfer-encoding") == 0){
	       racpy(&pRequestStruct->transferencoding, value);
	       }
	       
	       
	 
	 //***********   Integer   ***********
	 if(strcasecmp(name, "content-length") == 0){
	       pRequestStruct->contentlength = atoi(value);
	       if(serverconf->maxrequestdata != 0 && 
	          pRequestStruct->contentlength > serverconf->maxrequestdata){
		       return HE_REQUEST_TOO_LARGE;
		       }
		  
	       else{
	            returnval = getRequestData(&pRequestStruct->data, socketd, pRequestStruct->contentlength);
		    if(returnval){
		    	free(name);
		    	free(value);
		    	free(request);
		    	return returnval;
		    	}
		    cout << "POST DATA: [" << pRequestStruct->data << "]" << endl;
		    }
	       }
	 if(strcasecmp(name, "keep-alive") == 0){
	       pRequestStruct->contentlength = atoi(value);
	       }
	      
	 free(name);
	 free(value);
	 }
	
	 
	 
	 
     free(request);
     return 0;
     }    
        
